﻿/*------------------------------------------------------------------------------
* ProcessExecutor.h : プロセス実行クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#ifndef PROCESSEXECUTOR_H
#define PROCESSEXECUTOR_H

#include <QProcess>
#include <QTextCodec>
#include <QMessageBox>
#include <QMainWindow>

/**プロセス実行クラス
 * @brief The ProcessExecutor class
 */
class ProcessExecutor : public QObject
{
    Q_OBJECT

public:
    ProcessExecutor();
    ~ProcessExecutor();

    void initialize(QMainWindow *window);
    void start(QString prog, QStringList args);
    bool isRunning();

    void warning(QByteArray titleByte, QByteArray messageByte, QStringList &listDetail);

public slots:
    void slotStdError();
    void slotProcError(QProcess::ProcessError error);

private:
    /** 画面クラスへのポインタ
     * @brief mainWindow
     */
    QMainWindow *mainWindow;

    /** プロセスクラス
     * @brief process
     */
    QProcess process;

    /** 実行したプログラム
     * @brief progName
     */
    QString progName;

    /** 実行した引数
     * @brief progArgs
     */
    QStringList progArgs;
};

#endif // PROCESSEXECUTOR_H
