﻿/*------------------------------------------------------------------------------
* AntLibBase.ccp : ANTLib基底クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#include "AntLibBase.h"

using namespace std;

// static
int AntLibBase::logLevel = 0;
ostream* AntLibBase::logStream = NULL;


/** コンストラクタ
 * @brief AntLibBase::AntLibBase
 */
AntLibBase::AntLibBase()
{
}

/** デストラクタ
 * @brief AntLibBase::AntLibBase
 */
AntLibBase::~AntLibBase()
{
}

/** ログ出力
 * @brief AntLibBase::outputLog
 * @param level 出力レベル
 * @param msg 出力メッセージ
 * @param pos 発生個所
 */
void AntLibBase::outputLog(int level, string msg, string pos) {
    traceLog(level, msg, pos);
}

/** ログ出力
 * @brief AntLibBase::outputLog
 * @param msg 出力メッセージ
 * @param val 値
 * @param pos 発生個所
 */
void AntLibBase::outputLog(int level, string msg, double val, string pos) {
    ostringstream oStr;
    oStr << msg << val;
    traceLog(level, oStr.str(), pos);
}

/** ログ出力(時刻)
 * @brief AntLibBase::outputLog
 * @param msg 出力メッセージ
 * @param time 時刻
 * @param pos 発生個所
 */
void AntLibBase::outputLog(int level, string msg, gtime_t time, string pos) {
    ostringstream oStr;
    oStr << msg << time_str(time, 0);
    traceLog(level, oStr.str(), pos);
}

/** ログ出力(ベクトル)
 * @brief AntLibBase::outputVector
 * @param msg 出力メッセージ
 * @param vec ベクトル
 * @param num 要素数
 * @param precision 表示桁数
 * @param pos 発生個所
 */
void AntLibBase::outputVector(int level, string msg, const double* vec, int num, int precision, string pos) {
    ostringstream oStr;
    oStr << msg;
    for (int i = 0; i < num; i++) {
        oStr << ((i != 0)? ",": "") << setprecision(precision) << vec[i];
    }
    traceLog(level, oStr.str(), pos);
}

/** ログレベルの設定
 * @brief AntLibBase::setLogLevel
 * @param level レベル
 */
void AntLibBase::setLogLevel(int level) {
    logLevel = level;
}

/** Logストリームの設定
 * @brief AntLibBase::setStream
 * @param fstr ストリームへのポインタ
 */
void AntLibBase::setStream(std::ostream *ostr) {
    logStream = ostr;
}

/** Logストリームのクリア
 * @brief AntLibBase::clearStream
 */
void AntLibBase::clearStream() {
    logStream = NULL;
}

/** ログ出力
 * @brief AntLibBase::traceLog
 * @param level
 * @param msg
 * @param pos
 */
void AntLibBase::traceLog(int level, std::string msg, std::string pos) {
    if (logStream == NULL) {
        // ストリームが設定されていないなら何もしない
        return;
    }
    if (level > logLevel) {
        // ログレベルより高ければ何もしない
        return;
    }

    // ログ出力
    *logStream << time_str(getNowTime(), 0) << " " << pos << " - " << msg << endl;
}
