﻿/*------------------------------------------------------------------------------
* AntennaBase.h : アンテナ基底クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#ifndef ANTENNABASE_H
#define ANTENNABASE_H

#include <string>
#include <vector>
#include <sstream>
#include <iomanip>
#include <algorithm>
#include "AntLibBase.h"
#include "ObservationManager.h"
#include "NavigationManager.h"
#include "AntLibException.h"

#define RECEIVER_NO_TARGET      1   // 測定局のアンテナNo(rover)
#define RECEIVER_NO_REFERENCE   2   // 基準局のアンテナNo(reference)



/** アンテナ基底クラス
 * @brief The AntennaBase class
 */
class AntennaBase : public AntLibBase
{
public:
    AntennaBase();
    virtual ~AntennaBase();

    virtual void initialize(ObservationManager *obsMng, NavigationManager *navMng);
    virtual void reset();

    void setPosition(double x, double y, double z);
    void readRinex(std::vector<std::string> *vecFilepath, std::vector<std::string> *vecNavFilepath, char targetCode = '\0');
    std::vector<std::string> getPseudorangeCode(int freq);
    std::vector<std::string> getPhaseCode(int freq);
    double* calculatePosition(prcopt_t *opt = NULL);

    // getter
    double* getPosition();


protected:
    /** 受信機No
     * @brief receiverNo
     */
    int receiverNo;

    /** 観測データ管理クラス
     * @brief observationManager
     */
    ObservationManager *observationManager;

    /** 航法データ管理クラス
     * @brief navigationManager
     */
    NavigationManager *navigationManager;

    /** 局位置設定フラグ(ture：設定済み)
     * @brief positionFlag
     */
    bool positionFlag;

    /** アンテナ中心位置/アンテナ位相中心(ECEF,[m])
     * @brief position
     */
    double position[3];

    /** 観測データに含まれるコード(擬似距離)
     * @brief vecPseudorangeCode
     */
    std::vector<std::string> vecPseudorangeCode[NFREQ];

    /** 観測データに含まれるコード(搬送波位相)
     * @brief vecPhaseCode
     */
    std::vector<std::string> vecPhaseCode[NFREQ];
};

#endif // ANTENNABASE_H
