﻿/*------------------------------------------------------------------------------
* AntennaCalibrationsWriter.h : アンテナ特性出力クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#ifndef ANTENNACALIBRATIONSWRITER_H
#define ANTENNACALIBRATIONSWRITER_H

#include <sstream>
#include <iomanip>
#include <vector>
#include <map>
#include "time.h"
#include "FormatWriter.h"
#include "rtklib.h"

/** ファイルタイプ
 * @brief The FileType enum
 */
enum FileType {
    NGS_PCV = 1,
    ANTEX
};

/** ANTEX用パラメータ
 * @brief The paramForAntex_t struct
 */
struct paramForAntex_t {
    /** Name of agency
     * @brief agency
     */
    std::string agency;

    /** 方位角の刻み角[deg]
     * @brief dazi
     */
    double dazi;

    /** 天頂角の開始角[deg]
     * @brief zen1
     */
    double zen1;

    /**  天頂角の終了角[deg]
     * @brief zen2
     */
    double zen2;

    /** 天頂角の刻み角[deg]
     * @brief dzen
     */
    double dzen;
};

/** 方位角別のデータ
 * @brief The pcvAzDataForAntex_t struct
 */
struct pcvAzDataForAntex_t {
    /** 方位角
     * @brief az
     */
    double az;

    /** 天頂角におけるPCV
     * @brief vecPcv
     */
    std::vector<double> vecPcv;
};


/** アンテナ特性出力クラス
 * @brief The AntennaCalibrationsWriter class
 */
class AntennaCalibrationsWriter
{
public:
    AntennaCalibrationsWriter();
    virtual ~AntennaCalibrationsWriter();

    void open(std::string filename);
    void close();

    void outputOneCalibrationsForPcv(sta_t* sta, pcv_t* pcv, struct tm *createTime, std::string agency);
    void outputOneCalibrationsForAntex(sta_t* sta, pcv_t* pcv,
                                       struct tm *createTime, paramForAntex_t &param,
                                       std::vector<std::string> *vecFreqData);

    // static
    static std::vector<std::string> createFreqDataForAntex(pcv_t* pcv, int useSys, int freqNum,
                                                                  std::vector<pcvAzDataForAntex_t> *vecAzData,
                                                                  std::map<double, double> *mapNoAzData);
    static std::string toDateForAntex(struct tm *dateTime);

private:
    void outputDateForPcv(struct tm *dateTime);
    void outputDateForAntex(gtime_t gtime);

    static std::string outputFreqNameForAntex(int useSys, int useFreq);

    /** 書式出力クラス
     * @brief formatWriter
     */
    FormatWriter formatWriter;
};

#endif // ANTENNACALIBRATIONSWRITER_H
