﻿/*------------------------------------------------------------------------------
* MultipathData.ccp : マルチパス結果保管補完クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#include "MultipathData.h"

using namespace std;

/** コンストラクタ
 * @brief MultipathData::MultipathData
 */
MultipathData::MultipathData()
{
}


/** コピーコンストラクタ
 * @brief MultipathData::MultipathData
 * @param obj
 */
MultipathData::MultipathData(const MultipathData &obj)
{
    time = obj.time;
    sat = obj.sat;
    azel[0] = obj.azel[0];
    azel[1] = obj.azel[1];
    mp = obj.mp;
    amp = obj.amp;
}

/** デストラクタ
 * @brief MultipathData::~MultipathData
 */
MultipathData::~MultipathData()
{
}

/** 単純移動平均算出
 * @brief MultipathData::calculateAMP
 * @param vecMultipathData 単純移動平均を行うマルチパスデータ
 * @param startIdx 開始INDEX
 * @param endIdx 終了INDEX
 * @param range 移動平均の対象データ数
 */
void MultipathData::calculateAMP(vector<MultipathData> &vecMultipathData, int startIdx, int endIdx, int range) {
    if (startIdx > endIdx) {
        // 開始/終了INDEXの関係が不正
        throw AntLibException("startIdx is larger than endIdx.", "MultipathData::calculateAMP");
    }

    int num = endIdx - startIdx;
    for (int i = 0; i <= num; i++) {
        if (i == 0) {
            // 一つ目はそのまま保存
            vecMultipathData[startIdx + i].amp = vecMultipathData[startIdx + i].mp;
            continue;
        }

        // 単純平均のための和を計算
        int sIdx = ((i - range) < 0)? 0: (i - range);
        int eIdx = ((i + range) > num)? num: (i + range);
        double sum = 0.0;
        for (int j = startIdx + sIdx; j <= startIdx + eIdx; j++) {
            sum += vecMultipathData[j].mp;
        }

        // 単純平均を保存
        vecMultipathData[startIdx + i].amp = sum / (eIdx - sIdx + 1);
    }
}
