﻿/*------------------------------------------------------------------------------
* NavErrWriter.ccp : 観測誤差モデルテーブル出力クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#include "NavErrWriter.h"

using namespace std;


/** コンストラクタ
 * @brief NavErrWriter::NavErrWriter
 */
NavErrWriter::NavErrWriter()
{
}

/** デストラクタ
 * @brief NavErrWriter::~NavErrWriter
 */
NavErrWriter::~NavErrWriter()
{
    close();
}

/** ファイルオープン
 * @brief NavErrWriter::open
 * @param filename ファイル名
 */
void NavErrWriter::open(string filename) {
    // ファイルのオープン(追記)
    formatWriter.open(filename, true);
}

/** ファイルクローズ
 * @brief NavErrWriter::close
 */
void NavErrWriter::close() {
    // ファイルのクローズ
    formatWriter.close();
}

/** 観測誤差モデルの出力
 * @brief NavErrWriter::write
 * @param rcvName 受信機名
 * @param antName アンテナ名
 * @param data 観測誤差モデル
 */
void NavErrWriter::write(string rcvName, string antName, estNavErrorData_t data) {

    // 受信機名
    formatWriter.write(20, rcvName);
    formatWriter.writeSpace(2);

    // アンテナ名
    formatWriter.write(20, antName);
    formatWriter.writeSpace(2);

    // 衛星システムの略称
    if (data.sys == SYS_GPS) {
        formatWriter.write(3, "GPS");
    } else if (data.sys == SYS_GLO) {
        formatWriter.write(3, "GLO");
    } else if (data.sys == SYS_QZS) {
        formatWriter.write(3, "QZS");
    } else {
        formatWriter.write(3, "ALL");
    }
    formatWriter.writeSpace(2);

    // 観測データコード
    formatWriter.write(3, data.code);
    formatWriter.writeSpace(2);

    // 係数a
    formatWriter.write(5, 3, data.estCoef[0]);
    formatWriter.writeSpace(2);

    // 係数b
    formatWriter.write(5, 3, data.estCoef[1]);

    formatWriter.flush();
}
