﻿/*------------------------------------------------------------------------------
* NavErrWriter.h : 観測誤差モデルテーブル出力クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#ifndef NAVERRWRITER_H
#define NAVERRWRITER_H

#include "rtklib.h"
#include "FormatWriter.h"

/** 推定した観測モデル
 * @brief The estNavErrorData_t struct
 */
struct estNavErrorData_t {
    /** 衛星システム
     * @brief sys
     */
    int sys;

    /** 観測データコード
     * @brief code
     */
    std::string code;

    /** 係数a/b[m]
     * @brief a
     */
    double estCoef[2];
};

/** 観測誤差モデルテーブル出力クラス
 * @brief The NavErrWriter class
 */
class NavErrWriter
{
public:
    NavErrWriter();
    ~NavErrWriter();

    void open(std::string filename);
    void close();
    void write(std::string rcvName, std::string antName, estNavErrorData_t data);

private:
    /** 書式出力クラス
     * @brief formatWriter
     */
    FormatWriter formatWriter;

};

#endif // NAVERRWRITER_H
