﻿/*------------------------------------------------------------------------------
* PeoridData.h : 期間データクラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#ifndef PEORIDDATA_H
#define PEORIDDATA_H

#include <iostream>
#include <vector>
#include "rtklib.h"
#include "AntLibException.h"

/** 期間データ
 * @brief The PeoridData class
 */
class PeoridData
{
public:
    PeoridData();
    ~PeoridData();

    void setPeorid(gtime_t sTime, gtime_t eTime);
    void setData(gtime_t dataTime, double value);
    bool contain(gtime_t dataTime);
    double getData(gtime_t dataTime);
    std::vector<double>* getAllData();

    double linear(int index, gtime_t dataTime);

private:
    /** データ時刻
     * @brief vecData
     */
    std::vector<gtime_t> vecTime;

    /** データ
     * @brief vecData
     */
    std::vector<double> vecData;

    /** データ期間の開始時刻
     * @brief startDataTime
     */
    gtime_t startTime;

    /** データ期間の終了時刻
     * @brief endTime
     */
    gtime_t endTime;
};

#endif // PEORIDDATA_H
