﻿/*------------------------------------------------------------------------------
* ResultWriter.ccp : 結果出力クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#include "ResultWriter.h"

using namespace std;

/** コンストラクタ
 * @brief ResultWriter::ResultWriter
 */
ResultWriter::ResultWriter()
{
    delimiter = RESULT_WRITER_DELMITER;
}

/** デストラクタ
 * @brief ResultWriter::~ResultWriter
 */
ResultWriter::~ResultWriter()
{
    close();
}

/** 区切り文字設定
 * @brief ResultWriter::setDelimiter
 * @param str 区切り文字
 */
void ResultWriter::setDelimiter(std::string str) {
    delimiter = str;
}

/** 文字列出力(一行)
 * @brief ResultWriter::writeLine
 * @param str 文字列
 */
void ResultWriter::writeLine(string str) {
    if (!isOpen()) {
        // 開いてないなら何もしない
        return;
    }

    outputStream << str << endl;
}

/** データ出力(一行)
 * @brief ResultWriter::writeLine
 * @param vec データ列(数値)
 */
void ResultWriter::writeLine(vector<double> &vec) {
    writeLine(vec, 0, (int)vec.size(), 0);
}

/** データ出力(一行)
 * @brief ResultWriter::writeLine
 * @param vec データ列(数値)
 * @param startIdx 開始位置
 * @param vecNum データ数
 * @param precision 精度
 */
void ResultWriter::writeLine(vector<double> &vec, int startIdx, int vecNum, int precision) {
    if (!isOpen()) {
        // 開いてないなら何もしない
        return;
    }

    for (int i = 0; i < vecNum; i++) {
        int idx = i + startIdx;
        if (precision > 0) {
            // 桁指定あり
            outputStream << setprecision(precision)
                        << ((i != 0)? delimiter: "") << vec[idx];
        } else {
            outputStream << ((i != 0)? delimiter: "") << vec[idx];
        }
    }
    outputStream << endl;
}

/** データ出力(一行、時刻付き)
 * @brief ResultWriter::writeLine
 * @param gtime 時刻
 * @param vec　データ列(数値)
 * @param precision 精度
 */
void ResultWriter::writeLine(gtime_t gtime,vector<double> &vec, int precision) {
    if (!isOpen()) {
        // 開いてないなら何もしない
        return;
    }

    outputStream << time_str(gtime, 2);
    for (int i = 0; i < (int)vec.size(); i++) {
        if (precision > 0) {
            // 桁指定あり
            outputStream << setprecision(precision) << delimiter << vec[i];
        } else {
            outputStream << delimiter << vec[i];
        }
    }
    outputStream << endl;
}

/** データ出力(一行)
 * @brief ResultWriter::writeLine
 * @param arrayVal データ列(数値)
 * @param dataNnum データ数
 * @param precision 精度
 */
void ResultWriter::writeLine(double* arrayVal, int dataNum, int precision) {
    vector<double> vec;
    for (int i = 0; i < dataNum; i++) {
        vec.push_back(arrayVal[i]);
    }
    writeLine(vec, 0, (int)vec.size(), precision);
}

/** flush処理
 * @brief ResultWriter::flush
 */
void ResultWriter::flush() {
    outputStream.flush();
}
