﻿/*------------------------------------------------------------------------------
* WriterBase.ccp : ファイル出力基底クラス
*
*    Copyright (C) 2014 by Geospatial Information Authority of Japan,
*    All rights reserved.
*    
*    Released under the GPL Licenses.
*
*
*  Original software: ANTTOOL ver.2.1
*
*    Copyright(c) 2007-2011 by T.Takasu, all rights reserved.
*
*
* references :
*
* history : 2015/01/08  1.0  new
*-----------------------------------------------------------------------------*/

#include "WriterBase.h"

using namespace std;

/** コンストラクタ
 * @brief WriterBase::WriterBase
 */
WriterBase::WriterBase()
{
}

/** デストラクタ
 * @brief WriterBase::~WriterBase
 */
WriterBase::~WriterBase()
{
    close();
}

/** ファイルオープン
 * @brief WriterBase::open
 * @param filename ファイル名(パス含む)
 * @param append true：追記、false：上書き
 */
void WriterBase::open(string filename, bool append) {
    // ファイルオープン
    if (append) {
        // (追記モード)
        outputStream.open(filename.c_str(), ios::out | ios::app );
    } else {
        // (上書きモード)
        outputStream.open(filename.c_str(), ios::out | ios::trunc );
    }
}

/** ファイルクローズ
 * @brief WriterBase::close
 */
void WriterBase::close() {
    if (isOpen()) {
        // 開いていたらクローズ
        outputStream.close();
    }
}

/** ファイルのオープンチェック
 * @brief WriterBase::isOpen
 * @return true：ファイルオープン
 */
bool WriterBase::isOpen() {
    return outputStream.is_open();
}

/** 出力ストリーム取得
 * @brief WriterBase::getStream
 * @return 出力ストリーム(ファイルがオープンされていない場合は標準出力)
 */
ostream* WriterBase::getStream() {
    if (!isOpen()) {
        return &cout;
    }
    return &outputStream;
}
